##################################################################################
##                                                                              ##
##                              Better Camera V 1.0                             ##
##                              by Lamberto Tedaldi                             ##
##                              Officine Pixel S.n.c                            ##
##                              www.ffx.it                                      ##
##                              www.officinepixel.com                           ##
##                                                                              ##
##################################################################################

extends KinematicBody
export var max_distance = 15
export var camera_height = 6
export var camera_distance = 10
export(NodePath) var target_object=null
#export var far =500
var camera_position_vect = [Vector3(0,1,-1), Vector3(0,1,1), Vector3(-1,1,0), Vector3(1,1,0)]
export(int, "Above Back", "Above Front", "Above Right", "Above Left") var camera_position_id = 0
var object_name=""
var camera_ray = null
var object = null
var positionTargetObject = null

func _ready():
	if target_object == null:
		OS.alert("Please set a 'Target Object' for " + get_name(), "ERROR")
		get_tree().quit()
		return
	set_fixed_process(true)
	camera_ray = get_node("RayCast")	# this ray is used for checking the height of ground under the camera
	camera_ray.add_exception( self )
	object = get_node(target_object)
	object_name = object.get_name()
	var delta = Vector2(camera_height, camera_distance).length()
	if delta > max_distance:
		max_distance = delta + 1
	positionTargetObject = Position3D.new()
	object.add_child(positionTargetObject)
	positionTargetObject.set_translation(Vector3(camera_position_vect[camera_position_id].x*camera_distance, camera_position_vect[camera_position_id].y*camera_height, camera_position_vect[camera_position_id].z*camera_distance))
	#get_node("Camera").set_zfar(far)

	

func _fixed_process(delta):
		# -------------------------- rotation
	var target  = object.get_global_transform().origin + Vector3(0,2,0)
	var up = Vector3(0,1,0)
	get_node("Camera").look_at(target,up)
	
	
	# -------------------------- position
	var pos = get_global_transform().origin
	var p = positionTargetObject.get_global_transform().origin
	var positionTarget = Vector3(p.x, p.y, p.z)
	if(camera_ray.is_colliding()):
		if object.get_global_transform().origin.y < camera_ray.get_collision_point().y + camera_height :
			positionTarget = Vector3(p.x, camera_ray.get_collision_point().y + camera_height, p.z)
		else:
			positionTarget = Vector3(p.x, object.get_global_transform().origin.y+camera_height, p.z)
	else:
		positionTarget = Vector3(p.x, p.y, p.z)
		
	var distance = abs((positionTarget - pos).length())
	pos = pos + ((positionTarget - pos)/15)
	move_to(pos)
	
	# ------------------------- if camera stucks somewhere let's reset the view if object is far enough
	if distance>max_distance:
		var space_state = get_world().get_direct_space_state()
		var result = space_state.intersect_ray( positionTarget, object.get_global_transform().origin , [self])
		if result.size()!=0:
			if(result.collider.get_name() == object_name):
				if( my_can_move_to( positionTarget ) ):
					set_collide_with_static_bodies(false)
				else:
					set_collide_with_static_bodies(false)
					move_to(positionTarget)
	else:
		set_collide_with_static_bodies(true)
	

func my_can_move_to(pos):
	var space_state = get_world().get_direct_space_state()
	var starting_point = get_global_transform().origin
	var result = space_state.intersect_ray( starting_point, pos )
	if(result.size()==0):
		return true
	else:
		return false


